﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

using Infragistics.Controls.Menus;
using Infragistics.Windows;

namespace xamMenuFun
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {        
        #region Fields

        private bool _buttonClicked = false;
        private XamMenuItem _lastClicked = null;
        private List<HierarchicalElementLocation> _locations = new List<HierarchicalElementLocation>();

        #endregion Fields

        #region Constructors

        public MainWindow()
        {
            InitializeComponent();            
        }

        #endregion Constructors

        #region Methods

        private void WireExpanderEvents(Grid grid, bool wire)
        {
            foreach (UIElement child in grid.Children)
            {
                Expander expander = child as Expander;
                if (expander != null)
                {
                    if (wire)
                    {
                        // Wire event to capture when expanders are expanded/collapsed
                        expander.SizeChanged += new SizeChangedEventHandler(expander_SizeChanged);
                    }
                    else
                    {
                        // Unwire event to capture when expanders are expanded/collapsed
                        expander.SizeChanged -= new SizeChangedEventHandler(expander_SizeChanged);
                    }

                    // ASSUMPTION: There can be two and only two levels of collapsable panels. 
                    //             Therefore only need to check this expander for child expanders
                    //             and no need for recursion.                    
                    object[] childExpanders = UIElementTreeHelper.FindControls(expander, typeof(Expander), 0);
                    if (childExpanders != null && childExpanders.Length > 0)
                    {
                        // Get the Grid the child expanders are part of
                        Grid childGrid = expander.Content as Grid;
                        if (childGrid != null)
                        {
                            foreach (Expander childExpander in childExpanders)
                            {
                                if (wire)
                                {
                                    // Wire event to capture when expanders are expanded/collapsed
                                    childExpander.SizeChanged += new SizeChangedEventHandler(expander_SizeChanged);
                                }
                                else
                                {
                                    // Unwire event to capture when expanders are expanded/collapsed
                                    childExpander.SizeChanged -= new SizeChangedEventHandler(expander_SizeChanged);
                                }
                            }
                        }
                    }
                }
            }
        }

        /// <summary>
        /// Get the location of all expanders.
        /// </summary>
        /// <param name="grid"></param>
        private void ProcessLocationData(Grid grid)
        {
            double location = 0;
            List<HierarchicalElementLocation> childLocations = null;

            foreach (UIElement child in grid.Children)
            {
                Expander expander = child as Expander;
                if (expander != null)
                {
                    // ASSUMPTION: There can be two and only two levels of collapsable panels. 
                    //             Therefore only need to check this expander for child expanders
                    //             and no need for recursion.                    
                    object[] childExpanders = UIElementTreeHelper.FindControls(expander, typeof(Expander), 0);
                    
                    if (childExpanders != null && childExpanders.Length > 0)
                    {
                        // Get the Grid the child expanders are part of
                        Grid childGrid = expander.Content as Grid;
                        if (childGrid != null)
                        {
                            childLocations = new List<HierarchicalElementLocation>();
                            foreach (Expander childExpander in childExpanders)
                            {
                                RowDefinition childRowDef = childGrid.RowDefinitions[Grid.GetRow(childExpander)];
                                // The child element uses the same location as its parent. The offset will be used to position.
                                HierarchicalElementLocation childLocation = new HierarchicalElementLocation(childExpander.Name, location, childRowDef.Offset);
                                childLocations.Add(childLocation);
                            }
                        }
                    }

                    RowDefinition rowDef = grid.RowDefinitions[Grid.GetRow(expander)];
                    _locations.Add(childLocations == null ? new HierarchicalElementLocation(expander.Name, location, rowDef.Offset) : new HierarchicalElementLocation(expander.Name, location, rowDef.Offset, childLocations));
                    location += rowDef.ActualHeight;

                    // Child locations might have been set by a prior Expander, reset to null
                    childLocations = null;
                }
            }
        }

        private void ResetAllPanelLocations()
        {
            Grid svGrid = _scrollViewer.Content as Grid;
            if (svGrid != null)
            {
                _locations.Clear();
                ProcessLocationData(svGrid);
            }
        }

        private int GetLocationIndex()
        {
            int index = 0;
            for (int i = 0; i < _locations.Count; i++)
            {
                if (_scrollViewer.VerticalOffset >= _locations[i].Location)
                    index = i;
                else
                    break;
            }

            return index;
        }

        #endregion Methods

        #region Event handlers

        void expander_SizeChanged(object sender, SizeChangedEventArgs e)
        {
            // Need to reset all panel locations
            ResetAllPanelLocations();
        }

        /// <summary>
        /// Not currently wired to anything
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void xamNavigator_ItemClicked(object sender, Infragistics.Controls.Menus.ItemClickedEventArgs e)
        {
            //XamMenuItem mi = e.Item;
            //mi.IsSubmenuOpen = false;
            GroupPanelData item = e.Item.Header as GroupPanelData;
            
            MessageBox.Show("Item clicked: " + item.Title);
        }

        private void NavigatorItemLabel_Loaded(object sender, RoutedEventArgs e)
        {
            XamMenuItem xmi = Utilities.GetAncestorFromType(sender as DependencyObject, typeof(XamMenuItem), true) as XamMenuItem;

            // Set tooltip
            TextBlock textBlock = sender as TextBlock;
            ToolTipService.SetInitialShowDelay(textBlock, 1000);            
            textBlock.ToolTip = textBlock.Text;

            // Wire events -- click means navigate to group panel, SubmenuOpened is to keep drop down from happening
            xmi.Click += new EventHandler(xmi_Click);
            xmi.SubmenuOpened +=new EventHandler(xmi_SubmenuOpened);
            xmi.SubmenuClosed += new EventHandler(xmi_SubmenuClosed);
            

        }

        void xmi_SubmenuClosed(object sender, EventArgs e)
        {
            _buttonClicked = false;
        }

        void xmi_Click(object sender, EventArgs e)
        {
            XamMenuItem mi = sender as XamMenuItem;

            // Only if we've clicked something new
            if (_lastClicked == null || (_lastClicked != null && _lastClicked != mi))
            {
                // Need to decorate the clicked menu item to indicate it is active                               
                DecorateSelectedMenuItem(mi);

                if (_lastClicked != null)
                {
                    // Reset prior selection
                    _lastClicked.FontWeight = System.Windows.FontWeights.Normal;
                }

                _lastClicked = mi;
                
                //// Scroll to correct spot
                GroupPanelData item = mi.Header as GroupPanelData;
                //MessageBox.Show("Clicked: " + item.Title + " location: [" + item.ParentLocationIndex.ToString() + "," + item.LocationIndex.ToString() + "]");

                HierarchicalElementLocation parentLocation = null;
                HierarchicalElementLocation childOffset = null;

                if (item.HasParent)
                {
                    parentLocation = _locations[item.ParentLocationIndex];
                    childOffset = parentLocation.Children[item.LocationIndex];
                    _scrollViewer.ScrollToVerticalOffset(parentLocation.Location + childOffset.Offset);
                }
                else
                {
                    // Don't be confused, this is a parent but use its LocationIndex.
                    parentLocation = _locations[item.LocationIndex];
                    _scrollViewer.ScrollToVerticalOffset(parentLocation.Location);
                }                
            }
        }

        private void DecorateSelectedMenuItem(XamMenuItem mi)
        {
            mi.FontWeight = System.Windows.FontWeights.Bold;                                    
        }

        void xmi_SubmenuOpened(object sender, EventArgs e)
        {            
            XamMenuItem mi = sender as XamMenuItem;
            if (mi.HasChildren && !_buttonClicked)
            {                
                mi.IsSubmenuOpen = false;
                xmi_Click(sender, e);
            }

            // Reset the button flag if it was true this go around
            if (_buttonClicked)
            {
                _buttonClicked = false;
            }
        }

        private void Button_Click(object sender, RoutedEventArgs e)
        {
            // Need a way to keep the xmi_SubmenuOpened event from processing
            // This is klugey way for now.
            _buttonClicked = true;

            // Drop down submenu
            XamMenuItem xmi = Utilities.GetAncestorFromType(sender as DependencyObject, typeof(XamMenuItem), true) as XamMenuItem;
            
            if (xmi.HasChildren)
            {
                xmi.IsSubmenuOpen = true;
            }
        }

        private void NavigatorSubitem_Loaded(object sender, RoutedEventArgs e)
        {
            XamMenuItem xmi = Utilities.GetAncestorFromType(sender as DependencyObject, typeof(XamMenuItem), true) as XamMenuItem;

            // Set tooltip
            TextBlock textBlock = sender as TextBlock;
            ToolTipService.SetInitialShowDelay(textBlock, 1000);
            textBlock.ToolTip = textBlock.Text;

            // Wire events -- click means navigate to group panel, SubmenuOpened is to keep drop down from happening
            xmi.Click += new EventHandler(xmi_Click);
        }

        private void _scrollViewer_Loaded(object sender, RoutedEventArgs e)
        {
            if (this.IsLoaded)
            {
                ScrollViewer scrollViewer = sender as ScrollViewer;
                Grid svGrid = scrollViewer.Content as Grid;

                // Going to want to use the RowDefinition for the location
                // as the Expander is padded at top. Need a complex object
                // since expanders can be nested.
                if (svGrid != null)
                {
                    // Process all Expander locations, after method finishes
                    // execution, the _location variable will have all
                    // the Expander locations.
                    ProcessLocationData(svGrid);

                    // Wire expander events
                    WireExpanderEvents(svGrid, true);
                }
            }
        }

        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            // Need to wire here after the scroll viewer has been loaded
            _scrollViewer.SizeChanged += new SizeChangedEventHandler(_scrollViewer_SizeChanged);
            _scrollViewer.ScrollChanged += new ScrollChangedEventHandler(_scrollViewer_ScrollChanged);
        }

        private void _scrollViewer_SizeChanged(object sender, SizeChangedEventArgs e)
        {
            if (_emptyPanel != null)
                _emptyPanel.Height = _scrollViewer.ViewportHeight;
        }

        private void _scrollViewer_ScrollChanged(object sender, ScrollChangedEventArgs e)
        {
            int locationIndex = GetLocationIndex();
            XamMenu menu = _navigator;
            XamMenuItem xmi = Utilities.GetDescendantFromType(menu, typeof(XamMenuItem), true) as XamMenuItem;            
            
        }

        #endregion Event handlers
    }
}
